
#include <stdlib.h>
#include <math.h>

#include "cgdk.h"
#include "svga.h"
#include "b256_.h"
#include "lin.h"
#include "timer.h"

#ifndef CGDK_TRIAL

struct TB256C
{
  TB256C *next;
  void *dst;
  short  dstFormatX;
  void *spr;
  short  sprFormatX;
  short  sizeX;
  short  sizeY;
  int  type;
};

static TB256C **B256_bank;
static int B256_bankMax;
static TB256C *B256C;
static int B256C_last;
static int B256C_maxCached;

void B256C_done (void)
{
  if (B256C)
  {
    _free (B256C);
    B256C = NULL;
  }
  if (B256_bank)
  {
    _free (B256_bank);
    B256_bank = NULL;
    B256_bankMax = 0;
  }
  B256C_maxCached = 0;
  B256C_last = 0;
}

void B256C_init (void)
{
  int i;
  int bankMax;

  B256C_done ();
  B256C_maxCached = 3*SVGAC_maxCached;
  B256C = (TB256C *) _alloc (B256C_maxCached * sizeof (TB256C));
  bankMax = _div (SVGA_scanLines * SVGA_virtualX, 65536L)+1;
  B256_bankMax = bankMax;
  B256_bank = (TB256C * *) _alloc (sizeof (TB256C * *) * bankMax);

  for (i = 0; i < bankMax; i++)
  {
    B256_bank [i] = NULL;
    B256C [i].sizeY = 0;
  }

  B256C_last = bankMax;
}

static void B256C_putSpr (int bank, void *dst, int dstFormatX,
                            void *spr, int sprFormatX, int sizeX, int sizeY)
{
  int pos;

  if (sizeY)
  {
    if (B256_bank [bank] == NULL)
    {
      pos = bank;
      B256_bank [bank] = B256C+bank;
    }
    else
    {
      pos = B256C_last;
      B256_bank [bank]->next = B256C+pos;
      B256C_last++;
      B256_bank [bank] = B256C+pos;
    }
    B256C [pos].next = NULL;
    B256C [pos].dst = dst;
    B256C [pos].dstFormatX = dstFormatX;
    B256C [pos].spr = spr;
    B256C [pos].sprFormatX = sprFormatX;
    B256C [pos].sizeX = sizeX;
    B256C [pos].sizeY = sizeY;
    B256C [pos].type = 1;
  }
}

static void B256C_putWin (int bank, void *dst, int dstFormatX,
                            void *spr, int sprFormatX, int sizeX, int sizeY)
{
  int pos;

  if (sizeY)
  {
    if (B256_bank [bank] == NULL)
    {
      pos = bank;
      B256_bank [bank] = B256C+bank;
    }
    else
    {
      pos = B256C_last;
      B256_bank [bank]->next = B256C+pos;
      B256C_last++;
      B256_bank [bank] = B256C+pos;
    }
    B256C [pos].next = NULL;
    B256C [pos].dst = dst;
    B256C [pos].dstFormatX = dstFormatX;
    B256C [pos].spr = spr;
    B256C [pos].sprFormatX = sprFormatX;
    B256C [pos].sizeX = sizeX;
    B256C [pos].sizeY = sizeY;
    B256C [pos].type = 0;
  }
}

void B256C_flushBank (int i)
{
  TB256C *vesac;

  if (B256_bank [i] == NULL)
    return;

  VESA_setBank (i);

  vesac = B256C+i;

  do
  {
    if (vesac->type)
    {
      LIN_movSpr_ (vesac->dst, vesac->dstFormatX, vesac->spr, vesac->sprFormatX, vesac->sizeX, vesac->sizeY);
    }
    else
    {
      LIN_movWin_ (vesac->dst, vesac->dstFormatX, vesac->spr, vesac->sprFormatX, vesac->sizeX, vesac->sizeY);
    }
    vesac = vesac->next;
  }
  while (vesac);

  B256_bank [i] = NULL;
}

void B256C_flush (void)
{
  int i;
  int pos;
  int visible;
  int visibleEnd;
  int retrace;
  int beamY;
  double beamRest;
  int bankEndY;
  double retraceRest;
  int retraceTime;
  int retraceDetect;

  /*
  B256C_last = B256_bankMax;

  retrace = TIMER_retraceTime / TIMER_retraceTimeMaxF;
  CGDK_fp++;

//  lastBank = _div (SVGA_virtualPool, 65536L)+1;

  pos = SVGA_curXNext + SVGA_curYNext * SVGA_virtualX;
  visibleStart = hi (pos);

  pos += SVGA_max;
  visibleEnd = hi (pos)+1;
  if (lo (pos) != 0)
  {
    visibleEnd++;
  }

  visible = visibleStart;
  if (visible == 0)
  {
    invisible = visibleEnd;
  }
  else
  {
    invisible = 0;
  }

  while (retraceRest = fmod (TIMER_retraceTime, TIMER_retraceTimeMaxF),
         retraceRest < TIMER_retraceTimeMaxF)
  {
    CGDK_fp+=5;
    beamY = (SVGA_maxY * (retraceRest-TIMER_retraceTimeMaxF01)) / (TIMER_retraceTimeMaxF);

    retraceTime = TIMER_retraceTime;

    while ((TIMER_retraceTime == retraceTime) &&
           (retraceDetect = SVGA_retraceDetect (), !retraceDetect) )
    {

    }
  }
  */

  B256C_last = B256_bankMax;
//  i = 0;
//  CGDK_fp+=2;

  if (SVGA_enableRetrace)
  {
    pos = SVGA_curX + SVGA_curY * SVGA_virtualX;
    visible = hi (pos);
  //  visibleEnd = visible+2;

  //  retraceTime = TIMER_retraceTime;
  //  retraceDetect = 0;

    while (fmod (TIMER_retraceTime, TIMER_retraceTimeMaxF) < 0.666*TIMER_retraceTimeMaxF)
      ;

    // screen must be larger then 640 x 400
    B256C_flushBank (visible);
    B256C_flushBank (visible+1);
  }

  SVGA_wait (1);

/*  while (retraceRest = fmod (TIMER_retraceTime, TIMER_retraceTimeMaxF),
         retraceRest < TIMER_retraceTimeMaxF)
  {
    beamRest = (retraceRest-TIMER_retraceTimeMaxF01) / TIMER_retraceTimeMaxF;
    beamY = SVGA_maxY * beamRest;
    CGDK_fp+=5;

    retraceTime = TIMER_retraceTime;
    while ((TIMER_retraceTime == retraceTime) &&
           (retraceDetect = SVGA_retraceDetect (), !retraceDetect) )
    {
      if (visible == visibleEnd)
      {
        break;
      }

      bankEndY = ((visible+1) * 65536) / SVGA_virtualX - SVGA_curYPrev;

      if (bankEndY < beamY)
      {
        B256C_flushBank (visible);
        visible++;
      }
    }
    if (visible == visibleEnd || retraceDetect)
    {
      break;
    }
  }

  if (visible == visibleEnd)
  {
    SVGA_wait (1);
  }
  else
  {
    retraceTime = TIMER_retraceTime;
    while (TIMER_retraceTime - retraceTime < (0.1*TIMER_retraceTimeMaxF))
    {
      if (SVGA_retraceDetect ())
      {
        SVGA_wait (1);
        break;
      }
    }
  } */

  for (i = 0; i < B256_bankMax; i++)
  {
    B256C_flushBank (i);
  }

//  SVGA_scroll ();
}

#endif
