
#include <stdlib.h>

#include "svga.h"
#include "mir.h"
#include "spr.h"

#include "keyb.h"

struct TMIRC
{
  TMIR *mir;
  short  x;
  short  y;
  short  startY;
  short  startX;
  int  value;
};

static TMIRC *MIRC;
static int MIRC_last;
static int MIRC_maxCached;

void MIRC_done (void)
{
  if (MIRC)
  {
    _free (MIRC);
    MIRC = NULL;
  }
  MIRC_maxCached = 0;
  MIRC_last = 0;
}

void MIRC_init (int maxCached)
{
  MIRC_done ();
  MIRC_maxCached = maxCached;
  MIRC = (TMIRC *) _alloc (maxCached * sizeof (TMIRC));
}

void MIR_update (TMIR *mir, int x, int y)
{
  TMIRC *mirc;
  int mirx, miry;
  int i;
  int status;

  if (mir)
  {
    status = mir->status;
    if ((status & MIR_STATUS_UPDATE) == 0)
    {
      if (status & MIR_STATUS_CACHE)
      {
        i = 0;
        mirc = MIRC;
        while (mirc->mir != mir)
        {
          i++;
          mirc++;
        }
      }
      else
      {
        i = MIRC_last;
      }

      if (i < MIRC_maxCached)
      {
        mirc = MIRC+i;

        if (i == MIRC_last)
        {
          mir->status |= MIR_STATUS_CACHE;
          mirc->mir = mir;
          MIRC_last++;
        }

        mirc->x = x;
        mirc->y = y;

        if ((status & MIR_STATUS_RESTORE) == 0)
        {
          if (mir->y < y)
            y =  mir->y;
          if (mir->x < x)
            x =  mir->x;
        }

        mirc->startY = y;
        mirc->startX = x;

        if (mir->type & MIR_TYPE_SHADEOWN)
          y += 30000;

        mirc->value = y;
      }
      else
      {
        MIRC_update (mir, x, y);
      }
//      mir->status |= MIR_STATUS_UPDATE;
    }
  }
}


static int MIRC_comp (const void *src, const void *dst)
{
  return ((TMIRC *)src)->value - ((TMIRC *)dst)->value;
}

static void MIRC_insertSort (void)
{
  int i,j;
  int startMir;
  TMIR *mir;
  TMIRC tmirc;
  TMIRC *mirc, *mirc2;
  int sx1, sx2, sy1, sy2;
  int dx1, dx2, dy1, dy2;

  startMir = MIRC_last;
  while ((startMir > 0) && (MIRC [startMir-1].mir->type & MIR_TYPE_SHADEOWN))
  {
    startMir--;
  }

//  if (KEYB_pressed (KEY_SPACE))
//    abs (0);

  for (i = startMir; i < MIRC_last; i++)
  {
    mirc = MIRC+i;
    mir = mirc->mir;
    sy1 = mirc->startY;
    sy2 = mirc->y+mir->sizeY;
    sx1 = mirc->startX;
    sx2 = mirc->x+mir->sizeX;

    for (j = i; j > 0; j--)
    {
      mirc2 = mirc;
      mirc--;
      mir = mirc->mir;
      dy1 = mirc->startY;
      if (sy1 >= dy1)
        break;
      dy2 = mirc->y+mir->sizeY;
      dx1 = mirc->startX;
      dx2 = mirc->x+mir->sizeX;
      if (SPR_touch (sx1, sy1, sx2, sy2, dx1, dy1, dx2, dy2))
        break;

      tmirc = *mirc2;
      *mirc2 = *mirc;
      *mirc = tmirc;
    }
  }
}

void MIRC_flush (void)
{
  int i;
  TMIR *mir;
  TMIRC *mirc;

  qsort (MIRC, MIRC_last, sizeof (TMIRC), MIRC_comp);

  MIRC_insertSort ();

  mirc = MIRC;
  for (i = 0; i < MIRC_last; i++)
  {
    mirc->mir->status &= -1 ^ MIR_STATUS_CACHE;
    MIRC_update (mirc->mir, mirc->x, mirc->y);
    mirc++;
  }
  MIRC_last = 0;
}
