
#include "cgdk.h"
#include "svga.h"
#include "keyb.h"
#include "timer.h"

// class point

class point
{
  protected:
    int x, y;

  public:
    int getX (void);
    int getY (void);
    void place (int newX, int newY);
    point ();
};


point::point ()
{
  x = y = 0;
}

int point::getX (void)
{
  return x;
}

int point::getY (void)
{
  return y;
}

void point::place (int newX, int newY)
{
  point::x = newX;
  point::y = newY;
}


// class pixel

class pixel:public point
{
  protected:
    char color;

  public:

  virtual void draw (int posX, int posY, char initColor);
  virtual void redraw (char newColor);
  virtual void hide (void);
};

void pixel::redraw (char newColor)
{
  color = newColor;

  SVGA_putPixelNoVSB (getX (), getY (), color);
}

void pixel::hide (void)
{
  redraw (0);
}


void pixel::draw (int posX, int posY, char initColor)
{
  place (posX, posY);

  redraw (initColor);
}

// class square

class square:public pixel
{
  protected:
    int len;

  public:
    virtual void draw (int posX, int posY, int length, char initColor);
    square ();
    virtual void redraw (char newColor);
};

void square::redraw (char newColor)
{
  int i;

  for (i = 0; i <= len; i++)
  {
    SVGA_putPixelNoVSB (x + i, y, newColor);
    SVGA_putPixelNoVSB (x, y+i, newColor);
    SVGA_putPixelNoVSB (x + i, y+len, newColor);
    SVGA_putPixelNoVSB (x+len, y+i, newColor);
  }
}

square::square ()
{

}

void square::draw (int posX, int posY, int length, char initColor)
{
  place (posX, posY);

  len = length;

  redraw (initColor);
}



class filled:public square
{
  public:
    filled ();
    virtual void redraw (char newColor);
};

void filled::redraw (char newColor)
{
  int i, j;

  for (j = 0; j < len; j++)
  {
    for (i = 0; i <= j; i++)
    {
      SVGA_putPixelNoVSB (x + i, y, newColor);
      SVGA_putPixelNoVSB (x, y+i, newColor);
      SVGA_putPixelNoVSB (x + i, y+j, newColor);
      SVGA_putPixelNoVSB (x+j, y+i, newColor);
    }
  }
}

filled::filled ()
{

}


int main (void)
{
  pixel pix;
  square sq;
  filled fil;

  CGDK_init ();
  TIMER_init (TIMER_OLDFREQ_DEF);

  SVGA_setVideoMode (VGA_320x200x256);

  pix.draw (100, 100, 1);
  sq.draw (50, 20, 40, 2);
  fil.draw (150, 60, 60, 3);

  KEYB_wait ();

  pix.hide ();
  sq.hide ();
  fil.hide ();

  SVGA_clear (0);

  return 0;
}
