
/* VGA Low-Level rutines      (C) Cerberus Electronics 1995 */
/* ======================================================== */
/*                                                          */
/* Tagets : DOS4GW                                          */
/*                                                          */

#include <conio.h>
#include <dos.h>
#include <stdlib.h>
#include <mem.h>

#include "svga.h"
#include "dpmi.h"
#include "vga.h"
#include "keyb.h"
#include "timer.h"
#include "pal.h"
#include "tm.h"

#define VGA_GETVIDEOMODE 0x0F
#define VGA_TEST 0x1C

#define VGA_VERTICALRETRACE 1
#define VGA_HORIZONTALRETRACE 8

#define SEQU_ADDR 0x3C4

#include "l256_.cpp"


int CRTC_ADDR, STATUS_ADDR;

void *SVGA_videoBuf;
int VGA_installed;

static int VGA_getCRTCAddr (void)
{
  if (inp (0x3CC) & 1)
    return 0x3D4;
  else
    return 0x3B4;
}


static int VGA_getStatusAddr (void)
{
  if (inp (0x3CC) & 1)
    return 0x3DA;
  else
    return 0x3BA;
}

int SVGA_retraceDetect (void)
{
  return (inp (STATUS_ADDR) & VGA_HORIZONTALRETRACE);
}

void SVGA_waitVerticalRetrace (int count)
{
  int i;
  int port = STATUS_ADDR;

  for (i = 0; i < count; i++)
  {
    while (! (inp (port) & VGA_VERTICALRETRACE));
    while (  (inp (port) & VGA_VERTICALRETRACE));
  }
}

void SVGA_retraceStart (void)
{
  int port = STATUS_ADDR;
  while (! (inp (port) & VGA_HORIZONTALRETRACE))
    ;
}

void SVGA_retraceEnd (void)
{
  int port = STATUS_ADDR;
  while ((inp (port) & VGA_HORIZONTALRETRACE))
    ;
}

void SVGA_waitRetrace (int count)
{
  int i;

  for (i = 0; i < count; i++)
  {
    SVGA_retraceStart ();
    SVGA_retraceEnd ();
  }
}

void SVGA_wait (int count)
{
  int i;

  for (i = 0; i < count; i++)
  {
    KEYB_update ();

    if (SVGA_scroll != XVGA_scroll)
    {
      SVGA_retraceStart ();
      SVGA_scroll ();
      SVGA_retraceEnd ();
    }
    else
    {
      SVGA_retraceStart ();
      SVGA_retraceEnd ();
      SVGA_scroll ();
    }

    TIMER_retraceReprogram ();
//    TIMER_retraceTrap++;
  }
}

/*
void SVGA_waitRevert (int count)
{
  int i;

  for (i = 0; i < count; i++)
  {
    SVGA_retraceEnd ();
    SVGA_timer++;
    KEYB_update ();
    SVGA_retraceStart ();
  }
}
*/


void VGA_init (void)
{
  union REGS r;

  CRTC_ADDR = VGA_getCRTCAddr ();
  STATUS_ADDR = VGA_getStatusAddr ();

  SVGA_videoBuf = MK_DP (VGA_SEG, 0);
  r.h.ah = VGA_TEST;
  r.h.al = 0;
  r.x.ecx = 1;
  int386 (VGA_INT, &r, &r);
  VGA_installed =  (r.h.al == VGA_TEST);
}

int SVGA_returnVideoMode (void)
{
  union REGS r;

  if (SVGA_compatible < VESA_COMPATIBLE)
  {
    r.h.ah = VGA_GETVIDEOMODE;
    int386 (VGA_INT, &r, &r);
    return r.h.al;
  }
  else
  {
    return VESA_returnVideoMode ();
  }
}

int VGA_getModeInfo (int videoMode, int *xRes, int *yRes,
                        int *bytesPerPixel,int *bitsPerPixel)
{
  if (SVGA_compatible & VGA_COMPATIBLE)
  {
    switch (videoMode)
    {
      case VGA_40x25x16TC:
      case VGA_40x25x16TM:
      {
        *xRes = 40;
        *yRes = 25;
        *bytesPerPixel = 2;
        *bitsPerPixel = 16;
        break;
      }
      case VGA_80xXXx16TM:
      {
        if (TM_getVGA50Lines () )
          *yRes = 50;
        else
          *yRes = 25;
        *xRes = 80;

        *bytesPerPixel = 2;
        *bitsPerPixel = 16;
        break;
      }
      case VGA_80x50x16TM:
      {
        *xRes = 80;
        *yRes = 50;
        *bytesPerPixel = 2;
        *bitsPerPixel = 16;
        break;
      }
      case VGA_80x25x16TC:
      case VGA_80x25x16TM:
      {
        *xRes = 80;
        *yRes = 25;
        *bytesPerPixel = 2;
        *bitsPerPixel = 16;
        break;
      }
      case VGA_80x25x10TM:
      {
        *xRes = 80;
        *yRes = 25;
        *bytesPerPixel = 2;
        *bitsPerPixel = 10;
        break;
      }
      case VGA_320x200x16:
      {
        *xRes = 320;
        *yRes = 200;
        *bytesPerPixel = 1;
        *bitsPerPixel = 4;
        break;
      }
      case VGA_640x200x16:
      {
        *xRes = 640;
        *yRes = 200;
        *bytesPerPixel = 1;
        *bitsPerPixel = 4;
        break;
      }
      case VGA_640x350x16:
      {
        *xRes = 640;
        *yRes = 350;
        *bytesPerPixel = 1;
        *bitsPerPixel = 4;
        break;
      }
      case VGA_640x480x16:
      {
        *xRes = 640;
        *yRes = 480;
        *bytesPerPixel = 1;
        *bitsPerPixel = 4;
        break;
      }
      case VGA_320x200x256:
      {
        *xRes = 320;
        *yRes = 200;
        *bytesPerPixel = 1;
        *bitsPerPixel = 8;
        break;
      }
      default:
      {
        return 1;
      }
    }
    return 0;
  }
  return 1;
}

void *SVGA_getVideoBuf (int mode)
{
  if (mode == VGA_80x25x10TM)
  {
    return MK_DP (VGA_SEGMONO, 0);
  }
  else
  {
    if (mode <= VGA_TEXTMODE)
    {
      return MK_DP (VGA_SEGTEXT, 0);
    }
    else
    {
      return MK_DP (VGA_SEG, 0);
    }
  }

}

void VGA_switchVideoMode (int mode)
{
  union REGS r;
  int m;

  if (mode == VGA_80xXXx16TM)
  {
    if (TM_getVGA50Lines () )
    {
      mode = VGA_80x50x16TM;
    }
    else
    {
      mode = VGA_80x25x16TM;
    }
  }

  if (mode == VGA_80x50x16TM)
    m = VGA_80x25x16TM;
  else
    m = mode;

  r.x.eax = m;
  int386 (VGA_INT, &r, &r);

  if (mode == VGA_80x50x16TM)
  {
    TM_setVGA50Lines ();
  }
}

int VGA_setVideoMode (int mode)
{
  int bytes,bits;

  SVGA_graph = 0;

  if (SVGA_compatible & VGA_COMPATIBLE)
  {
    if (mode <= VGA_MAXMODE)
    {
      SVGA_palType = PAL_6BIT;

      if (mode > VGA_TEXTMODE)
      {
        SVGA_graph = 1;
      }
      SVGA_videoBuf = SVGA_getVideoBuf (mode);

      SVGA_waitRetrace (1);
      PAL_clear (0, 256);
      SVGA_waitRetrace (2);

      VGA_switchVideoMode (mode);

      if (mode <= VGA_320x200x256)
      {
        SVGA_linearSeg = 0;
        #ifdef __TURBOC__
          SVGA_linearBuf = SVGA_videoBuf;
          SVGA_linearOff = SVGA_videoBuf;
        #else
          SVGA_linearBuf = MK_FP (DPMI_getDS (), VGA_SEG * 16);
          SVGA_linearOff = (void *) (VGA_SEG * 16);
        #endif
        SVGA_linearOff2 = SVGA_linearOff;
      }
      else
      {
        SVGA_linearBuf = NULL;
        SVGA_linearSeg = 0;
        SVGA_linearOff = NULL;
        SVGA_linearOff2 = NULL;
      }

      VGA_getModeInfo (mode, &SVGA_maxX, &SVGA_maxY, &SVGA_bytesPerPixel, &SVGA_bitsPerPixel);
      SVGA_max = (unsigned) SVGA_maxX*SVGA_maxY;
      SVGA_virtualX = SVGA_maxX;
      SVGA_virtualY = SVGA_maxY;
      SVGA_virtual = (long) SVGA_virtualX*SVGA_virtualY;
      SVGA_doubleBuffer = 0;
      SVGA_palType = PAL_6BIT;
      SVGA_scanLines = SVGA_virtualY;
      SVGA_curX = SVGA_curY = 0;
      SVGA_curXNext = SVGA_curYNext = 0;
      SVGA_maxXPlus = SVGA_maxYPlus = 0;

      return 0;
    }
    else
    {
      return 2;
    }
  }
  return 1;
}


#define VGA_SAVERESTORESTATE 0x1C
#define VGA_STATE_GETBUFFERSIZE 0
#define VGA_STATE_SAVE 1
#define VGA_STATE_RESTORE 2
#define VGA_STATE_MASK 7

static int VGA_getSaveStateBufferLen (void)
{
  REGS regs;

  regs.h.ah = VGA_SAVERESTORESTATE;
  regs.h.al = VGA_STATE_GETBUFFERSIZE;
  regs.x.ecx = VGA_STATE_MASK;
  int386 (VGA_INT, &regs, &regs);
  if (regs.h.al == VGA_SAVERESTORESTATE)
  {
    return 64*regs.w.bx;
  }
  else
  {
    return 0;
  }
}


void *VGA_saveState (void)
{
  REGS regs;
  RMINFO rmi = {0};
  void *buf;
  int len;

  if (SVGA_compatible)
  {
    len = VGA_getSaveStateBufferLen ();
    if (len)
    {
      buf = DPMI_alloc (len);
      if (buf)
      {
        rmi.h.ah = VGA_SAVERESTORESTATE;
        rmi.h.al = VGA_STATE_SAVE;
	rmi.w.cx = VGA_STATE_MASK;
        rmi.w.bx = DP_OFF (buf);
        rmi.w.es = DP_SEG (buf);
        DPMI_int (VGA_INT, &rmi);
        if (rmi.h.al == VGA_SAVERESTORESTATE)
        {
          return buf;
        }
        else
          return NULL;
      }
      else
      {
        return NULL;
      }
    }
    else
    {
      return NULL;
    }
  }
  else
    return NULL;
}

void VGA_restoreState (void *stateBuf)
{
  RMINFO rmi = {0};

  if (SVGA_compatible)
  {
    if (stateBuf)
    {
      rmi.h.ah = VGA_SAVERESTORESTATE;
      rmi.h.al = VGA_STATE_RESTORE;
      rmi.w.cx = VGA_STATE_MASK;
      rmi.w.bx = DP_OFF (stateBuf);
      rmi.w.es = DP_SEG (stateBuf);
      DPMI_int (VGA_INT, &rmi);
      DPMI_free (stateBuf);
    }
  }
}

